%% SMARTS input file generator 
% This script is used to generate the SMARTS input file which simulates the
% hourly spectra of a given location. According to the SMARTS manual, two
% seasons are classified (winter and summer), and this script generates the
% corresponding input files with default SMARTS models for these two seasons
% to achieve an annual spectra generation.

% author: Yilong
% date: April, 2024

clear; clc

%% For the case in the northen hemisphere
winter = [1 2 3 10 11 12];                              % winter months (by default uses mid-latitdue winter (MLW)
summer = [4 5 6 7 8 9];                                 % summer months (by default uses mid-latitude summer (MLS)

prompt = {'GIve a name to your simulation:'};                           % template: Delft_winter_scan (a string containing the location and the season)
name = char(inputdlg(prompt));
if isempty(name); error('Please put in the name!'); end

prompt = {'For what season do you want to perform the simulation (winter or summer)?'};
season = char(inputdlg(prompt));
if isempty(season); error('Please put in the season!'); end

prompt = { 'Enter the year:',...
    'Enter Latitude (positive in northern hemisphere):',...
    'Enter Longitude (positive is east of prime meridian):',...
     'Enter the time zone:'};
dlgtitle = 'Location of interest';
dims = [1 60];
definput = {'2023','52.01','4.36','1'};                                                    % Delft
answers = inputdlg(prompt,dlgtitle,dims,definput);
if isempty(answers); error('Please put in the longitude or latitude!'); end

%% Default lines by using the default models in SMARTS
switch season
    case 'winter'
        data = {
            name;
            1;
            [1013.25, 0, 0];
            1;
            'MLW';
            1;
            1;
            1;
            370;
            0;
            'SRA_CONTL';                                                                                          % This may be changed according to your simulation purpose (refer to the manual)
            0;
            0.084;
            -1;
            0.2;
            0;
            [280, 4000, 1.0, 1366.1];
            2;
            [280, 4000, 0.5];
            7;                                                                                                                % number of outputs (equal to the number of elements in the next card)
            [1, 2, 3, 4, 5, 9, 10];                                                                                 % ouputs (refer to the manual to determine what outputs are needed)
            0;
            0;
            0;
            0;
            3
            };

    case 'summer'
        data = {
            name;
            1;
            [1013.25, 0, 0];
            1;
            'MLS';
            1;
            1;
            1;
            370;
            0;
            'SRA_CONTL';                                                                                   % This may be changed according to your simulation purpose (refer to the manual)
            0;
            0.084;
            -1;
            0.2;
            0;
            [280, 4000, 1.0, 1366.1];
            2;
            [280, 4000, 0.5];
            7;                                                                                                        % number of outputs (equal to the number of elements in the next card)
            [1, 2, 3, 4, 5, 9, 10];                                                                         % ouputs (refer to the manual to determine what outputs are needed)
            0;
            0;
            0;
            0;
            3
            };
end

% Define the time series
days = [31 28 31 30 31 30 31 31 30 31 30 31];   % non-leap year
hour = 0:1:23;                              % 0.5 in decimal hour stands for 00.30 which indicates the time period between 00.00 to 01.00

%% Open a file to write the input for winter scan
if strcmp(season,'winter') == 1

    fid = fopen([name '.txt'], 'w');
    % Loop through each cell in the array
    for i = 1:length(data)
        if ischar(data{i})
            fprintf(fid, '''%s''\n', data{i});
        elseif isnumeric(data{i})
            if numel(data{i}) == 1
                fprintf(fid, '%g\n', data{i});
            else
                fprintf(fid, '%g ', data{i});
                fprintf(fid, '\n');
            end
        end
    end

    for i = 1:length(winter)
        month = winter(i);
        day = 1:days(month);
        for j = day
            for k = 1:length(hour)
                data = {answers{1}, month, day(j), hour(k), answers{2}, answers{3}, answers{4}};
                for p = 1:length(data)
                    if ischar(data{p})
                        fprintf(fid, '%s ', data{p});
                    elseif isnumeric(data{p})
                        fprintf(fid, '%g ', data{p});
                    end
                end
                fprintf(fid, '\n');
            end
        end
    end
    fclose(fid);

elseif strcmp(season,'summer') == 1

    fid = fopen([name '.txt'], 'w');

    % Loop through each cell in the array
    for i = 1:length(data)
        if ischar(data{i})
            fprintf(fid, '''%s''\n', data{i});
        elseif isnumeric(data{i})
            if numel(data{i}) == 1
                fprintf(fid, '%g\n', data{i});
            else
                fprintf(fid, '%g ', data{i});
                fprintf(fid, '\n');
            end
        end
    end

    for i = 1:length(summer)
        month = summer(i);
        day = 1:days(month);
        for j = day
            for k = 1:length(hour)
                data = {answers{1}, month, day(j), hour(k), answers{2}, answers{3}, answers{4}};
                for p = 1:length(data)
                    if ischar(data{p})
                        fprintf(fid, '%s ', data{p});
                    elseif isnumeric(data{p})
                        fprintf(fid, '%g ', data{p});
                    end
                end
                fprintf(fid, '\n');
            end
        end
    end
    fclose(fid);

else
    error('Wrong season is provided (either put in summer or winter).');
end
