%% SMARTS output file processing
% This script is used to process the output file of SMARTS

% author: Yilong
% date: April, 2024

clear; clc

%% Load the output data
% Select the input files
% ####  two with extension .out.txt and the other two withextension .ext.txt   ####
[inputs,paths] = uigetfile('*.*',  'All Files (*.*)','MultiSelect','on');       
if numel(inputs) < 4; error('Select the output files for the whole year.'); end

idx = find(contains(inputs,'winter'));
if contains(inputs{idx(1)},'.out.txt') 
    file_time_w = inputs{indx(1)}; file_spectra_w = inputs{idx(2)};
else
    file_time_w = inputs{idx(2)}; file_spectra_w = inputs{idx(1)};
end

idx = find(contains(inputs,'summer'));
if contains(inputs{idx(1)},'.out.txt') 
    file_time_s = inputs{indx(1)}; file_spectra_s = inputs{idx(2)};
else
    file_time_s = inputs{idx(2)}; file_spectra_s = inputs{idx(1)};
end

%% Read the selected files

% ############## Open the overview file #############
data_time_w = fopen([paths,file_time_w], 'r');
if data_time_w == -1
    error('File could not be opened, check name or path.')
end

% Initialize an array 
lines_time_w = {};

% Read each line
while ~feof(data_time_w)
    tline = fgetl(data_time_w);
    if ischar(tline)                                % Check if the line read is valid
        lines_time_w{end+1} = tline;
    end
end

% Close the file
fclose(data_time_w);

%############# Open the spectra file ##############
data_spectra_w = readmatrix([paths,file_spectra_w]);

% Generate an array for indices
totalindices = length(data_spectra_w);
n = 1; currentIndex = 1;
indices = 1;
while currentIndex < totalindices
    if mod(n,2) == 1
        indices = [indices; indices(end) + 2001];
    else
        indices = [indices; indices(end) + 2];
    end
    currentIndex = indices(end);
    n = n + 1;
end
indices = reshape(indices,2,[])';

% Extract the wavelength (column1) and GHI data (column5)
wavelength = data_spectra_w(1:2002,1);
GHI_spectra_w = cell(length(indices),1); n = 1;
for i = 1:length(GHI_spectra_w)
    GHI_spectra_w{i} = data_spectra_w(indices(n,1):indices(n,2),5);
    n = n + 1;
end
clear data_spectra_w

% ############## Open the overview file (summer) #############
data_time_s = fopen([paths,file_time_s], 'r');
if data_time_s == -1
    error('File could not be opened, check name or path.')
end

% Initialize an array 
lines_time_s = {};

% Read each line
while ~feof(data_time_s)
    tline = fgetl(data_time_s);
    if ischar(tline)                                % Check if the line read is valid
        lines_time_s{end+1} = tline;
    end
end

% Close the file
fclose(data_time_s);

%############# Open the spectra file ##############
data_spectra_s = readmatrix([paths,file_spectra_s]);

% Generate an array for indices
totalindices = length(data_spectra_s);
n = 1; currentIndex = 1;
indices = 1;
while currentIndex < totalindices
    if mod(n,2) == 1
        indices = [indices; indices(end) + 2001];
    else
        indices = [indices; indices(end) + 2];
    end
    currentIndex = indices(end);
    n = n + 1;
end
indices = reshape(indices,2,[])';

% Extract the wavelength (column1) and GHI data (column5)
GHI_spectra_s = cell(length(indices),1); n = 1;
for i = 1:length(GHI_spectra_s)
    GHI_spectra_s{i} = data_spectra_s(indices(n,1):indices(n,2),5);
    n = n + 1;
end
clear data_spectra_s


%% Match the time with spectra data
% Loop for cases winter or summer
winter = [1 2 3 10 11 12];                              % winter months (by default uses mid-latitdue winter (MLW)
summer = [4 5 6 7 8 9];                                 % summer months (by default uses mid-latitude summer (MLS)
days = [31 28 31 30 31 30 31 31 30 31 30 31];   % non-leap year

% Define the patterns
pattern_day = '====================================================================================================';
pattern_night = 'ERROR #7';

% Find out the indices with patterns
a = regexp(lines_time_w,pattern_day); a(cellfun(@isempty, a)) = {0};
b = regexp(lines_time_w,pattern_night); b(cellfun(@isempty, b)) = {0};
a = cell2mat(a)'; b = cell2mat(b)'; 
match_ab = a + b; match_ab(match_ab==0) = [];

GHI_winter = [[repmat(winter(1),days(1),1),(1:days(1))'];[repmat(winter(2),days(2),1),(1:days(2))'];...
    [repmat(winter(3),days(3),1),(1:days(3))']; [repmat(winter(4),days(10),1),(1:days(10))'];...
    [repmat(winter(5),days(11),1),(1:days(11))']; [repmat(winter(6),days(12),1),(1:days(12))']];
GHI_winter = num2cell(GHI_winter);

d = 1; s = 1;
for i = 1:length(GHI_winter)
    currentday = cell(24,1);
    for j = 1:24
        if match_ab(d) == 4
            currentday{j} = 0;
            d = d + 1;
        elseif match_ab(d) + match_ab(d+1) == 2
            currentday{j} = GHI_spectra_w{s};
            d = d + 2; s = s + 1;
        end
        GHI_winter{i,3} = currentday;
    end
end

% ############# Find out the indices with patterns (summer) ################
a = regexp(lines_time_s,pattern_day); a(cellfun(@isempty, a)) = {0};
b = regexp(lines_time_s,pattern_night); b(cellfun(@isempty, b)) = {0};
a = cell2mat(a)'; b = cell2mat(b)'; 
match_ab = a + b; match_ab(match_ab==0) = [];

GHI_summer = [[repmat(summer(1),days(4),1),(1:days(4))'];[repmat(summer(2),days(5),1),(1:days(5))'];...
    [repmat(summer(3),days(6),1),(1:days(6))']; [repmat(summer(4),days(7),1),(1:days(7))'];...
    [repmat(summer(5),days(8),1),(1:days(8))']; [repmat(summer(6),days(9),1),(1:days(9))']];
GHI_summer = num2cell(GHI_summer);

d = 1; s = 1;
for i = 1:length(GHI_summer)
    currentday = cell(24,1);
    for j = 1:24
        if match_ab(d) == 4
            currentday{j} = 0;
            d = d + 1;
        elseif match_ab(d) + match_ab(d+1) == 2
            currentday{j} = GHI_spectra_s{s};
            d = d + 2; s = s + 1;
        end
        GHI_summer{i,3} = currentday;
    end
end

% Annual spectra data for the location of interest
GHI_tot = [GHI_winter(1:90,:);GHI_summer;GHI_winter(91:end,:)];     

%% Generate the AST for the whole year
AST = []; GHI_spectrum = []; m_day = [];
for i = 1:length(GHI_tot)
    currentGHI = GHI_tot{i,3};
    hours = find(cellfun(@numel,currentGHI)>1);
    AST = [AST;hours]; m_day = [m_day;numel(hours)];
    dayGHI = cell2mat(currentGHI(hours)');
    GHI_spectrum = [GHI_spectrum,dayGHI];
end

clearvars -except GHI_tot GHI_winter GHI_summer wavelength days summer winter AST GHI_spectrum m_day

save('Catania_SMARTS.mat','AST','GHI_spectrum','wavelength','m_day');                  % Update the name of the location